require File.expand_path("../amqp/version", __FILE__)

module Vx
  module Common
    module AMQP

      autoload :Config,     File.expand_path("../amqp/config",    __FILE__)
      autoload :Session,    File.expand_path("../amqp/session",   __FILE__)
      autoload :Consumer,   File.expand_path("../amqp/consumer",  __FILE__)
      autoload :CLI,        File.expand_path("../amqp/cli",       __FILE__)
      autoload :Formatter,  File.expand_path("../amqp/formatter", __FILE__)

      module Supervisor
        autoload :Threaded, File.expand_path("../amqp/supervisor/threaded", __FILE__)
      end

      autoload :Instrument, File.expand_path("../amqp/mixins/instrument",   __FILE__)

      extend self

      @@config  = Common::AMQP::Config.new
      @@session = Common::AMQP::Session.new

      def configure
        yield config
      end

      def config
        @@config
      end

      def session
        @@session
      end

      def open
        session.open
      end

      def open?
        session.open?
      end

      def close
        session.close
      end

      def shutdown
        Common::AMQP::Session.shutdown
        Vx::Common::AMQP::Supervisor::Threaded.shutdown
      end

      def shutdown?
        Common::AMQP::Session.shutdown?
      end

      def instrument(name, payload, &block)
        if config.debug?
          STDOUT.puts "name: #{name} payload: #{payload.inspect}"
        end

        if config && config.instrumenter
          config.instrumenter.instrument(name, payload, &block)
        else
          yield if block_given?
        end
      end

    end
  end
end
