require 'vx/common/rack/builder'

module Vx
  module Common
    module AMQP
      class Config

        attr_accessor :default_exchange_options, :default_queue_options,
          :default_publish_options, :default_exchange_type, :pool_timeout,
          :heartbeat, :spawn_attempts, :content_type, :instrumenter, :debug,
          :on_error, :builders

        def initialize
          reset!
        end

        def formatter
          Common::AMQP::Formatter
        end

        def default_exchange_name
          "amq.#{default_exchange_type}"
        end

        def debug?
          @debug || ENV['VX_COMMON_AMQP_DEBUG']
        end

        def use(target, middleware, *args)
          @builders[target].use middleware, *args
        end

        def reset!
          @default_exchange_type = :topic
          @pool_timeout          = 0.1
          @heartbeat             = 10

          @publishing_builder    = nil
          @recieving_builder     = nil
          @subscribing_builder   = nil

          @spawn_attempts        = 1

          @content_type          = 'application/json'

          @callbacks             = {}
          @instrumenter          = nil
          @on_error              = ->(e, env){ nil }

          @builders = {
            pub: Vx::Common::Rack::Builder.new,
            sub: Vx::Common::Rack::Builder.new
          }

          @default_exchange_options = {
            durable:     true,
            auto_delete: false
          }

          @default_queue_options = {
            durable:     true,
            autodelete:  false,
            exclusive:   false
          }

          @default_publish_options = {
            durable:     false
          }

        end

      end
    end
  end
end
